/*
  Fast hash table implementation
  Max Volkov (Floyd_Rose), Sevastopol - 2007
  Written for XFLib project
  Maito: max at xfhome dot org
*/

#ifndef __HASH_H__
#define __HASH_H__

#define XFSOFT

#include <string.h>
#include <stdio.h>

#define HASH_NULL 0

#define HASH_MAGIC 0x48534148

#define HASH_COPY_KEY    1
#define HASH_COPY_DATA   2
#define HASH_REPLACE_KEY 4

/* Debugging flag */
//#define HASH_DEBUG

/* Generic hash table structure */
typedef struct
 {
  int magic;
  int size;
  void **block;
 } hash_t;

/* Hash table element structure */
typedef struct
 {
  int magic;            /* Magic number */
  void *next;           /* Next block pointer */
  int keyhash;          /* Key hash sum */
  void *key;            /* Key */
  int keysize;          /* Key size in bytes */
  void *data;           /* Data */
  int datasize;         /* Data size in bytes */
  int attr;             /* Storage attributes */
 } hash_block;

extern int hash_alloc_counter;
extern int hash_alloc_sum;
extern int hash_free_counter;

typedef void* (*hash_user_alloc_t)(int);
typedef void (*hash_user_free_t)(void*);

/* User-defined memory manager (set to 0 to use default malloc/free) */
extern hash_user_alloc_t hash_user_alloc;
extern hash_user_free_t  hash_user_free;

hash_t* hash_create(int hash_size);
void hash_destroy(hash_t *hash);

void* hash_alloc(int size);
void hash_free(void *p);

char* hash_strdup(void *data);
hash_t* hash_add(hash_t *hash,void *key,int keysize,void *data,int datasize,int attr);
int hash_function(int hashsize,void *string,int size);
int hash_sum(void *string,int size);
void* hash_find(hash_t *hash,void *key,int keysize,int *datasize);
hash_t *hash_remove(hash_t *hash,void *key,int keysize);
hash_t *hash_dup(hash_t *hash,int hash_size);
hash_t *hash_resize(hash_t *hash,int hash_size);

#ifdef HASH_DEBUG
void hash_print(hash_t *hash);
void hash_info();
#endif

#endif


