/*
  XFLib - XF management library

  (C) Max Volkov, Sevastopol 2006-2008
  Mailto: max at xfhome dot org
*/

/* XFLib Header */

#ifndef _XF_H
#define _XF_H

#define XFSOFT

#include "xf_conf.h"

#include <stdio.h>
#include <wchar.h>

#define XFS(string) L##string

/* Format version */
#define XF XFS("1.0")

/* Version xflib-0.1.7 */
#define XF_VN 0
#define XF_SN 1
#define XF_RN 7

#define XF_TITLE  XFS("XFLib 0.1.7")
#define XF_AUTHOR XFS("Max Volkov (xfhome.org)")

#ifndef XF_TEST
#define XFLIB ((XF_VN << 24) | (XF_SN << 16) | (XF_RN << 8))
#else
#define XFLIB ((XF_VN << 24) | (XF_SN << 16) | (XF_RN << 8) | 1)
#endif

/* Max. node name/class/value length */
#ifndef XF_NODESIZE
#define XF_NODESIZE 4096
#endif

/* Max. stack size (in elements) */
#ifndef XF_STACKSIZE
#define XF_STACKSIZE 4096
#endif

/* Number of blocks in hash tables */
#ifndef XF_HASH_SIZE
#define XF_HASH_SIZE 20
#endif

/* Number of node childen for which we create hash table */
#ifndef XF_HASH_NODE_SIZE
#define XF_HASH_NODE_SIZE 10
#endif

/* Enable auto-resizing of hash tables,
   extend tables in 8 times */
#ifndef XF_AUTOHASH
#define XF_AUTOHASH 8
#endif

/* Size of input/output buffer */
#ifndef XF_IO_BUFFER
#define XF_IO_BUFFER 1024
#endif

#define XF_NULL (0)

#define false (0)
#define true  (1)

#ifdef __cplusplus
extern "C" {
#endif

typedef wchar_t xfChar;    /* XFLib default 16-bit wide Unicode character */

typedef void xfVoid; /* Void */

#ifndef XF_64BIT
typedef long xfInt;
#else
typedef long long int xfInt;
#endif

typedef xfInt xfBool;	   /* Boolean type (false/true) */
typedef char xfNChar;      /* Normal 8-bit character */

#define xfHash hash_t

/*** Node (symbolic map element) ***/
typedef struct _xfNode xfNode;
struct _xfNode
 {
  xfVoid *map;                  /* Map which includes the node */
  xfNode *parent;               /* Parent node (XF_NULL for global node) */
  xfChar *name;                 /* Node name */
  xfChar *Class;                /* Class name */
  xfChar *value;                /* Node value */
  xfNode *next;                 /* One node brother (next child of its parent)*/
  xfNode *prev;                 /* One node brother (last child of its parent)*/
  xfNode *firstChild;           /* First node child */
  xfNode *lastChild;            /* Last node child */
  xfInt numChildren;            /* Number of children of the node */
  xfVoid *hash;		     	/* Hash table for child names */
 };

/*** Security policy ***/
typedef struct
 {
  xfInt nodeLimit;              /* Maximal number of possible nodes in a doc */
  xfInt docSize;                /* Maximal document size (bytes) */
 } xfSecurity;

/*** MAP (internal XF document representation) ***/
typedef struct
 {
  xfInt magic;		        /* Magic number */
  xfNode *root;                 /* Root node (has no name/class/value) */
  xfChar *error;  	        /* Error message */
  xfInt lineno;                 /* Line which contains error */
  xfInt colno;                  /* Column which contains error */
  xfChar *header;	        /* Comment (to be added into XF files) */

  /* Security information */
  xfInt nodeNum;                /* Number of nodes in the document */
  xfSecurity *security;         /* Security policy (0 if no security) */

  /* Document generating options */
  xfInt o_encoding;             /* Encoding for file gereration */
  xfInt o_addInfo;              /* Add XFLib comment to documents */
  xfInt o_addComment;           /* Add user comment to documents */
  xfInt o_charCodes;            /* Output hex codes of non-ASCII chars */
 } xfMap;

/*** XF Data Model (XFDM) node iterator ***/
typedef struct
 {
  xfNode *node;                 /* Node to iterate */
  xfNode *lastNode;             /* Last returned node */
  xfInt started;                /* If iteration was started */
  xfInt counter;                /* Iteration node counter */
  xfInt level;                  /* Iteration level counter */
  xfInt recursive;              /* If iterator is recursive */
 } xfIterator;

extern xfChar xfTitle[];
extern xfChar xfAuthor[];

/* User-defined memory manager types */
typedef void* (*xfUserAlloc_t)(int);
typedef void (*xfUserFree_t)(void*);

#ifdef __cplusplus
};
#endif

/* XF error codes
   We don't use error code 1, because -1 is reserved */
#define XFE_OK		0		/* Success			   */
#define XFE_IO		2		/* Input/output error		   */
#define XFE_FORMAT	3		/* Incorrect XF document	   */
#define XFE_VERSION	4		/* Incompatible version		   */
#define XFE_MEMORY	5		/* Not enough memory		   */
#define XFE_LIMIT	6		/* Too many objects		   */
#define XFE_STATE	7		/* Something is in incorrect state */
#define XFE_DATA	8		/* Incorrect input data 	   */
#define XFE_HMMM	9		/* Hmmm... Unknown error...        */
#define XFE_CLASS      10               /* Incorrect document class        */

/* Supported document encodings */
#define XF_ENC_ASCII    0       /* 7-bit ASCII */
#define XF_ENC_UTF8     1       /* 8-bit Unicode UTF-8 */
#define XF_ENC_UTF16    2       /* 16-bit Unicode UTF-16 (big/little endian) */

/* Boolean constants */
#undef false
#undef true
#define false 0
#define true  1

/* Map structure magic (i like coffee;) */
#define XF_MAP_MAGIC 0xC0FFEE77

#define _XF_SYS /* */

#if (defined(_Windows) && defined(__DLL__))
  #define _XF_API __declspec(dllexport)
#else
  #if (defined(_WIN32) && defined(_USRDLL))
    #define _XF_API __declspec(dllexport)
  #else
    #define _XF_API /* */
  #endif
#endif

#endif

