/*
  XFLib - XF management library

  (C) Max Volkov, Sevastopol 2006-2008
  Mailto: max at xfhome dot org
*/

/*
  Hash table management routines
*/

#include "xf.h"

#include "xf_api.h"
#include "xf_sys.h"

#include "hash.h"

_XF_API xfHash*
xfCreateHash(xfInt hashSize)
{
 return (xfHash*)hash_create(hashSize);
}

_XF_API xfVoid
xfDestroyHash(xfHash *hash)
{
 hash_destroy((hash_t*)hash);
}

_XF_API xfHash*
xfResizeHash(xfHash *hash,xfInt hashSize)
{
 return (xfHash*)hash_resize((hash_t*)hash,hashSize);
}

_XF_API xfHash*
xfHashAdd(xfHash *hash,xfVoid *key,xfInt keySize,xfVoid *data,xfInt dataSize,xfInt attr)
{
 return (xfHash*)hash_add((hash_t*)hash,key,keySize,data,dataSize,attr);
}

_XF_API xfHash*
xfHashRemove(xfHash *hash,xfVoid *key,xfInt keySize)
{
 return (xfHash*)hash_remove((hash_t*)hash,key,keySize);
}

_XF_API xfVoid*
xfHashFind(xfHash *hash,xfVoid *key,xfInt keySize,xfInt *dataSize)
{
 return (xfVoid*)hash_find((hash_t*)hash,key,keySize,dataSize);
}


_XF_SYS xfHash*
xfHashAddNode(xfHash *hash,xfNode *node)
{
 xfChar *name;

 if (!node) return hash;
 return
   xfHashAdd(hash,node->name,(xfStrlen(node->name) + 1) * sizeof(xfChar),
             node,4,0);
}

_XF_SYS xfHash*
xfHashRemoveNode(xfHash *hash,xfNode *node)
{
 if (!node) return hash;

 return xfHashRemove(hash,node->name,
                     (xfStrlen(node->name) + 1) * sizeof(xfChar));
}

_XF_SYS xfNode*
xfHashFindNode(xfHash *hash,xfChar *name)
{
 xfNode **node;

 return xfHashFind(hash,name,(xfStrlen(name) + 1) * sizeof(xfChar),0);
}

_XF_SYS xfNode*
xfCreateNodeHash(xfNode *node,xfInt hashSize)
{
 xfNode *c;

 node->hash = (xfHash*) hash_create(hashSize);

 if (!node->hash) return XF_NULL;

 c = xfChild(node);

 while (c)
  {
   xfHashAddNode((xfHash*)node->hash,c);
   c = xfNext(c);
  }

 return node;
}


