/*
  XFLib - XF management library

  (C) Max Volkov, Sevastopol 2006-2008
  Mailto: max at xfhome dot org
*/

/*
  XF high level function layer
*/

#include "xf.h"
#include "xf_api.h"
#include "xf_sys.h"

/* Locale information */
xfMap *xfMsgList = XF_NULL;
xfNode *xfMsgPtr = XF_NULL;

/*
  Install user-defined memory manager
*/
_XF_API xfVoid
xfSetMemManager(xfUserAlloc_t alloc,xfUserFree_t free)
{
 xfUserAlloc = alloc;
 xfUserFree = free;
}

/*
  Use default XFLib memory manager (malloc/free)
*/
_XF_API xfVoid
xfResetMemManager()
{
 xfSetMemManager(XF_NULL,XF_NULL);
}

/*
  Enters path (starting with selected node)
*/
_XF_API xfNode*
xfEnterPath(xfNode *node,xfChar *path)
{
 xfChar *word,*source,*iSource;
 xfNode *curNode;
 xfInt shift;

 if (!node) return XF_NULL;

 word = (xfChar*)xfAlloc(XF_NODESIZE * sizeof(xfChar));
 if (!word) return XF_NULL;
 source = (xfChar*)xfAlloc(XF_NODESIZE * sizeof(xfChar));
 if (!source) { xfFree(word); return XF_NULL; }

 iSource = source;

 xfStrcpy(source,path);

 curNode = node;

 for (shift = 0;;)
  {
   source += shift;
   shift = xfParsePath(source,word);
   if (!*word) break;
   if (!shift) return XF_NULL;

   curNode = xfEnter(curNode,word);
   if (!curNode)
    { xfFree(word); xfFree(iSource); return curNode; }
  }

 xfFree(word);
 xfFree(iSource);

 return curNode;
}

/*
  Adds path to node
*/
_XF_API xfNode*
xfAddPath(xfNode *node,xfChar *path)
{
 xfChar *word,*Source,*iSource;
 xfNode *curNode;
 xfInt shift;

 if (!node) return XF_NULL;

 word = (xfChar*)xfAlloc(XF_NODESIZE * sizeof(xfChar));
 if (!word) return XF_NULL;
 Source = (xfChar*)xfAlloc(XF_NODESIZE * sizeof(xfChar));
 if (!Source) { xfFree(word); return XF_NULL; }

 iSource = Source;

 xfStrcpy(Source,path);

 curNode = node;

 for (shift = 0;;)
  {
   Source += shift;
   shift = xfParsePath(Source,word);
   if (!*word) break;
   if (!shift) { xfFree(word); xfFree(Source); return XF_NULL; }

   curNode = xfAdd(curNode,word,XF_NULL,XF_NULL);
   if (!curNode)
    { xfFree(word); xfFree(iSource); return curNode; }
  }

 xfFree(word);
 xfFree(iSource);

 return curNode;
}

/*
  Get node value by path
*/
_XF_API xfChar*
xfGetPathValue(xfNode *node,xfChar *path)
{
 return xfGet(xfEnterPath(node,path));
}

/*
  Set node value by path
*/
_XF_API xfChar*
xfSetPathValue(xfNode *node,xfChar *path,xfChar *value)
{
 return xfSet(xfEnterPath(node,path),value);
}

/*
  Get node value by class
*/
_XF_API xfChar*
xfGetPathClass(xfNode *node,xfChar *path)
{
 return xfGetClass(xfEnterPath(node,path));
}

/*
  Set node class by path
*/
_XF_API xfChar*
xfSetPathClass(xfNode *node,xfChar *path,xfChar *Class)
{
 return xfSetClass(xfEnterPath(node,path),Class);
}

/*
  Clear node (remove children)
*/
_XF_API xfVoid
xfClear(xfNode *node)
{
 xfNode *c,*n;

 if (!node) return;
 c = node->firstChild;

 while (c)
  {
   n = c->next;
   xfRemove(c);
   c = n;
  }
}

/*
  Get document version
*/

_XF_API xfChar*
xfGetDocVersion(xfMap *map)
{
 xfNode *n;

 n = xfEnterPath(map->root,XFS("xf version"));
 if (!n) return XF_NULL;

 return xfGet(n);
}

/*
  Set document version
*/

_XF_API xfChar*
xfSetDocVersion(xfMap *m,xfChar *value)
{
 xfSet(xfAddPath(m->root,XFS("xf version")),value);

 return value;
}

/*
  Get document class
*/

_XF_API xfChar*
xfGetDocClass(xfMap *map)
{
 xfNode *n;

 n = xfEnterPath(map->root,XFS("xf class"));
 if (!n) return XF_NULL;

 return xfGet(n);
}

/*
  Set document class
*/

_XF_API xfChar*
xfSetDocClass(xfMap *m,xfChar *value)
{
 xfSet(xfAddPath(m->root,XFS("xf class")),value);

 return value;
}

/*
  Get document comment
*/

_XF_API xfChar*
xfGetDocComment(xfMap *map)
{
 xfNode *n;

 n = xfEnterPath(map->root,XFS("xf comment"));
 if (!n) return XF_NULL;

 return xfGet(n);
}

/*
  Set document comment
*/

_XF_API xfChar*
xfSetDocComment(xfMap *m,xfChar *value)
{
 xfSet(xfAddPath(m->root,XFS("xf comment")),value);

 return value;
}

/*
  Get document encoding
*/

_XF_API xfInt
xfGetDocEncoding(xfMap *map)
{
 if (!map) return XF_NULL;

 return map->o_encoding;
}

/*
  Set document encoding
*/

_XF_API xfVoid
xfSetDocEncoding(xfMap *map,xfInt encoding)
{
 if (map) map->o_encoding = encoding;
}

/*
  Get document header
*/

_XF_API xfChar*
xfGetDocHeader(xfMap *map)
{
 return map->header;
}

/*
  Set document header (to be written as a usual XF comment)
*/

_XF_API xfChar*
xfSetDocHeader(xfMap *map,xfChar *header)
{
 if (map->header) xfFree(map->header);
 map->header = XF_NULL;

 map->header = xfStrdup(header);

 return map->header;
}

/*
  Set option: adding XFLib comment to generated documents
*/
_XF_API xfVoid
xfOptionAddInfo(xfMap *map,xfInt option)
{
 if (!map) return;
 map->o_addInfo = option;
}

/*
  Set option: adding user comment to generated documents
*/
_XF_API xfVoid
xfOptionAddComment(xfMap *map,xfInt option)
{
 if (!map) return;
 map->o_addComment = option;
}

/*
  Set option: output hex codes of non-ASCII characters
  (recommended to set if using ASCII output - otherwise
  non-ASCII characters can be lost)
*/
_XF_API xfVoid
xfOptionCharCodes(xfMap *map,xfInt option)
{
 if (!map) return;
 map->o_charCodes = option;
}

/*
  Create XFLib security police
*/
_XF_API xfSecurity*
xfCreateSecurity()
{
 xfSecurity *sec;

 sec = (xfSecurity*)xfAlloc(sizeof(xfSecurity));
 if (!sec) return XF_NULL;

 sec->nodeLimit = XF_NULL;
 sec->docSize = XF_NULL;

 return sec;
}

/*
  Destroy security policy
*/
_XF_API xfVoid
xfDestroySecurity(xfSecurity *security)
{
 if (security) xfFree(security);
}

/*
  Get number of nodes in the document
*/
_XF_API xfInt
xfGetDocNodeNum(xfMap *map)
{
 if (!map) return XF_NULL;
 return map->nodeNum;
}

/*
  Get document security policy
*/
_XF_API xfSecurity*
xfGetDocSecurity(xfMap *map)
{
 if (!map) return XF_NULL;
 return map->security;
}

/*
  Set document security policy
*/
_XF_API xfSecurity*
xfSetDocSecurity(xfMap *map,xfSecurity *security)
{
 if (!map) return XF_NULL;
 map->security = security;
 return security;
}

_XF_API xfVoid
xfResetDocSecurity(xfMap *map)
{
 if (!map) return;
 map->security = XF_NULL;
}

/*
  Get security rule: maximal number of nodes
*/
_XF_API xfInt
xfSecurityGetNodeLimit(xfSecurity *security)
{
 if (!security) return XF_NULL;
 return security->nodeLimit;
}

/*
  Set security rule: maximal number of nodes
*/
_XF_API xfVoid
xfSecuritySetNodeLimit(xfSecurity *security,xfInt nodeLimit)
{
 if (!security) return;
 security->nodeLimit = nodeLimit;
}

/*
  Get security rule: maximal document size (bytes)
*/
_XF_API xfInt
xfSecurityGetDocSize(xfSecurity *security)
{
 if (!security) return XF_NULL;
 return security->docSize;
}

/*
  Set security rule: maximal document size (bytes)
*/
_XF_API xfVoid
xfSecuritySetDocSize(xfSecurity *security,xfInt docSize)
{
 if (!security) return;
 security->docSize = docSize;
}

/*
  Check if we can add node to map (security attributes)
*/
_XF_API xfInt
xfCanAddNode(xfMap *map)
{
 if (!map->security) return true;
 if (!map->security->nodeLimit) return true;

 if (map->nodeNum < map->security->nodeLimit) return true;

 return false;
}

/*
  Load message list from file
*/
_XF_API xfInt
xfLoadMsgList(xfNChar *filename)
{
 xfMap *msg;
 xfNode *ptr;
 xfInt res;
 xfChar *Class;

 msg = xfCreate();
 if (!msg) return XFE_MEMORY;

 res = xfReadFile(msg,filename);
 if (res)
  {
   xfDestroy(msg);
   return res;
  }

 Class = xfGetDocClass(msg);
 if (!Class)
  {
   xfDestroy(msg);
   return XFE_CLASS;
  }

 ptr = xfEnter(msg->root,L"Messages");
 if (!ptr)
  {
   xfDestroy(msg);
   return XFE_CLASS;
  }

 if (xfMsgList) xfDestroy(xfMsgList);

 xfMsgList = msg;
 xfMsgPtr = ptr;

 return XFE_OK;
}

/*
  Load message list from string
*/
_XF_API xfInt xfLoadMsgListString(xfChar *string)
{
 xfMap *msg;
 xfNode *ptr;
 xfInt res;
 xfChar *Class;

 msg = xfCreate();
 if (!msg) return XFE_MEMORY;

 res = xfReadString(msg,string);

 if (res)
  {
   xfDestroy(msg);
   return res;
  }

 Class = xfGetDocClass(msg);
 if (!Class)
  {
   xfDestroy(msg);
   return XFE_CLASS;
  }

 ptr = xfEnter(msg->root,L"Messages");
 if (!ptr)
  {
   xfDestroy(msg);
   return XFE_CLASS;
  }

 if (xfMsgList) xfDestroy(xfMsgList);

 xfMsgList = msg;
 xfMsgPtr = ptr;

 return XFE_OK;
}

/*
  Convert message code into currently supported language
*/
_XF_API xfChar*
xfTranslateMsg(xfChar *msg)
{
 xfChar *s;

 s = xfGet(xfEnter(xfMsgPtr,msg));

 /* If no message found, we return default message code string */
 if (s) return s;

 return msg;
}

/*
  Create iterator
*/
_XF_API xfIterator*
xfCreateIterator()
{
 xfIterator *s;

 s = (xfIterator*)xfAlloc(sizeof(xfIterator));

 if (!s) return XF_NULL;

 s->counter = s->level = 0;
 s->node = XF_NULL;
 s->lastNode = XF_NULL;
 s->started = false;

 return s;
}

/*
  Destroy iterator
*/
_XF_API xfVoid
xfDestroyIterator(xfIterator *iterator)
{
 if (!iterator) return;
 xfFree(iterator);
}

/*
  Start iteration
*/
_XF_API xfVoid
xfSetIteration(xfIterator *iterator,xfNode *node,xfInt recursive)
{
 if (!iterator) return;
 iterator->node = node;
 iterator->lastNode = XF_NULL;
 iterator->counter = iterator->level = 0;
 iterator->recursive = recursive;
 iterator->started = false;
}

/*
  Execute one iteration
*/
_XF_API xfNode*
xfNextIteration(xfIterator *iterator)
{
 if (iterator->recursive)
  {
   if (!iterator->started)
    {
     iterator->started = true;
     iterator->lastNode = xfChild(iterator->node);
     return iterator->lastNode;
    }
   else
    {
     if (xfChild(iterator->lastNode))
      { iterator->lastNode = xfChild(iterator->lastNode); iterator->level++;
        return iterator->lastNode; }

     while (!xfNext(iterator->lastNode))
      {
       if (!iterator->lastNode) return iterator->lastNode;
       if (iterator->lastNode == iterator->node) return XF_NULL;
       iterator->lastNode = xfParent(iterator->lastNode); iterator->level--;
      }

     if (iterator->lastNode == iterator->node) return XF_NULL;
     iterator->lastNode = xfNext(iterator->lastNode);
     iterator->counter++; return iterator->lastNode;
    }
  }
 else
  {
   if (!iterator->started)
    {
     iterator->started = true;
     iterator->lastNode = xfChild(iterator->node);
     return iterator->lastNode;
    }
   iterator->lastNode = xfNext(iterator->lastNode);
   if (iterator->lastNode) iterator->counter++;
   return iterator->lastNode;
  }

 return XF_NULL;
}

/*
  Get number of iterated items
*/
_XF_API xfInt
xfIterationCounter(xfIterator *iterator)
{
 return iterator->counter;
}

/*
  Get recursion level (for recursive iteration)
*/
_XF_API xfInt
xfIterationLevel(xfIterator *iterator)
{
 return iterator->level;
}

