/*
  XFLib - XF management library

  (C) Max Volkov, Sevastopol 2006-2008
  Mailto: max at xfhome dot org
*/

/*
  Library of very common functions (memory, strings, names etc)
*/

#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "xf.h"
#include "xf_api.h"
#include "xf_sys.h"

xfInt xfAllocCounter = 0;
xfInt xfFreeCounter = 0;
xfInt xfAllocSum = 0;

xfUserAlloc_t xfUserAlloc = XF_NULL;
xfUserFree_t  xfUserFree  = XF_NULL;

_XF_API xfInt
xfBigEndian()
{
 short w; char *c;

 c = (char*)&w; w = 0x1234;

 if (c[0] == 0x12) return true;
 return false;
}

_XF_API xfInt
xfLittleEndian()
{
 return xfBigEndian()? false: true;
}

_XF_API xfInt
xfStrlen(xfChar *s)
{
 return wcslen(s);
}

_XF_API xfChar*
xfStrcpy(xfChar *dst,xfChar *src)
{
 return wcscpy(dst,src);
}

_XF_API xfInt
xfStrcmp(xfChar *s1,xfChar *s2)
{
 return wcscmp(s1,s2);
}

_XF_API xfChar*
xfStrchr(xfChar *s,xfInt c)
{
 return wcschr(s,c);
}

_XF_API xfChar*
xfStrstr(xfChar *s1,xfChar *s2)
{
 return wcsstr(s1,s2);
}

/*
  Duplicates string
*/
_XF_API xfChar*
xfStrdup(xfChar *s)
{
 xfChar *_s;

 _s = (xfChar*)xfAlloc((xfStrlen(s) + 1) * sizeof(xfChar));
 if (!_s) return XF_NULL;

 xfStrcpy(_s,s);

 return _s;
}

_XF_API xfInt
xfNStrlen(xfNChar *s)
{
 return strlen(s);
}

_XF_API xfNChar*
xfNStrcpy(xfNChar *dst,xfNChar *src)
{
 return strcpy(dst,src);
}

_XF_API xfInt
xfNStrcmp(xfNChar *s1,xfNChar *s2)
{
 return strcmp(s1,s2);
}

_XF_API xfNChar*
xfNStrchr(xfNChar *s,xfInt c)
{
 return strchr(s,c);
}

_XF_API xfNChar*
xfNStrstr(xfNChar *s1,xfNChar *s2)
{
 return strstr(s1,s2);
}

/*
  Duplicates string
*/
_XF_API xfNChar*
xfNStrdup(xfNChar *s)
{
 xfChar *_s;

 _s = (xfChar*)xfAlloc((xfNStrlen(s) + 1) * sizeof(xfNChar));
 if (!_s) return XF_NULL;

 xfNStrcpy(_s,s);

 return _s;
}

/*
  Unpacks string
*/

xfChar*
xfUnpackString(xfChar *s)
{
 xfInt i,nl,a;
 xfChar *ns;
 xfInt escape = false,size;
 xfInt code,c;
 xfChar xdigit[]= L"0123456789ABCDEFabcdef";

 ns = (xfChar*)xfAlloc((xfStrlen(s) + 1) * sizeof(xfChar));
 if (!ns) return XF_NULL;
 nl = 0;

 memset(ns,'\0',(xfStrlen(s) + 1) * sizeof(xfChar));

 for (i = 0; s[i]; i++)
  {
   c = s[i];
   if (c == '\\')
    {
     if (!escape) { escape = true; size = 0; code = 0; }
     else { ns[nl] = '\\'; nl++; }
    }
   else
    {
     if (!escape) { ns[nl] = c; nl++; }
     else
      {
       if (xfStrchr(xdigit,c))
        { code <<= 4; code |= xfDigitHexToInt(c); size++; }
       else
        {
         if (c == ';')
          {
           escape = false;
           ns[nl] = code; nl++;
          }
         if (c == '\\') { escape = false; ns[nl] = '\\'; nl++; break; }
         if (c == '*')  { escape = false; ns[nl] = '\"'; nl++; break; }
         if (c == 'n')  { escape = false; ns[nl] = '\n'; nl++; break; }
         if (c == 's')  { escape = false; ns[nl] = ' ';  nl++; break; }
         if (c == 't')  { escape = false; ns[nl] = '\t'; nl++; break; }
        }
      }
    }
  }

 xfStrcpy(s, ns);
 xfFree(ns);

 return s;
}

/*
  Digit converter: base16 -> base10
*/
_XF_SYS xfInt
xfDigitHexToInt(xfInt hex)
{
 if (!isxdigit(hex)) return 0;
 if (hex >= '0' && hex <= '9') return hex - '0';
 if (hex >= 'a' && hex <= 'f') return hex - 'a' + 10;
 if (hex >= 'A' && hex <= 'F') return hex - 'A' + 10;
 /* UNREACHABLE */ return 0;
}

/* Checks if string needs to be packed */
_XF_SYS xfInt
xfNeedToPack(xfChar *s)
{
 xfInt i;

 if (!s) return 0;
 for (i = 0; s[i]; i++)
  {
   if (!isalnum(s[i]) && s[i] != '_' && s[i] != '-')
    return 1;
  }

 return 0;
}

/*
  Packs string
*/

_XF_SYS xfChar*
xfPackString(xfChar *s, xfChar *ns,int charCodes)
{
 xfInt i;
 xfInt enc,nl = 0;
 xfChar xdigit[] = L"0123456789abcdef";
 xfChar a;

 for (i = 0; s[i]; i++)
  {
   enc = false;
   if ((s[i] >= 0x0000) && (s[i] < 0x0020)
     || s[i] == 0x000A  ||  s[i] == 0x000D || s[i] == '"') enc = true;

   if (charCodes && (s[i] >= 0x0080)) enc = true;

   if (!enc) { ns[nl] = s[i]; nl++; }
   else
    {
     ns[nl] = '\\'; nl++;
     if (xfStrchr(L"\\\"\n\t",s[i]))
      {
       switch (s[i])
        {
         case '\\': ns[nl] = '\\'; break;
         case '\"': ns[nl] = '*'; break;
         case 0x000A: ns[nl] = 'n'; break;
         case '\t': ns[nl] = 't'; break;
        }
       nl++;
      }
     else
      {
       if (s[i] & 0xFF80)
        {
         a = (s[i] >> 12) & 0x0F;  ns[nl] = xdigit[a]; nl++;
         a = (s[i] >> 8)  & 0x0F;  ns[nl] = xdigit[a]; nl++;
         a = (s[i] >> 4)  & 0x0F;  ns[nl] = xdigit[a]; nl++;
         a =  s[i]        & 0x0F;  ns[nl] = xdigit[a]; nl++;
         ns[nl] = ';'; nl++;
        }
       else
        {
         a = (s[i] >> 4) & 0x0F;
         ns[nl] = xdigit[a]; nl++;
         a = s[i] & 0x0F;
         ns[nl] = xdigit[a]; nl++;
         ns[nl] = ';'; nl++;
        }
      }
    }
  }

 ns[nl] = 0;

 return ns;
}

/*
  Parses XF path string
*/

_XF_SYS xfInt
xfParsePath(xfChar *pathName,xfChar *nodeName)
{
 xfInt i;
 xfChar *word,s;
 xfInt string = 0,l;

 word = (char*)xfAlloc(XF_NODESIZE * sizeof(xfChar));
 if (!word) return XFE_MEMORY;

 memset(word,0,XF_NODESIZE * sizeof(xfChar));

 l = xfStrlen(pathName);
 i = 0;
 if (!l) { *nodeName = 0; xfFree(word); return 0; }

 while (!*word && i < l)
  {
   for (; pathName[i]; i++)
    {
     s = pathName[i];

     if (string)
      {
       if (s == '\"')
        {
         string = 0;
         break;
        }
       else
        {
         if (xfStrlen(word) + 1 < XF_NODESIZE)
          { word[xfStrlen(word)] = s; }
         }
       continue;
      }

     if (s == '\"')
      {
       string = 1;
       continue;
      }

     if (xfStrchr(L" \t\x0A\x0D",s)) break;

     if (!isalnum(s) && s != '_' && s != '-')
      { *nodeName = 0; xfFree(word); return 0; }

     if (xfStrlen(word) + 1 < XF_NODESIZE)
      { word[xfStrlen(word)] = s; }
    }
   if (i != l) i++;
  }

 xfStrcpy(nodeName, word);

 xfFree(word);

 return i;
}

/*
  Convert XFLib UTF-16 string to ASCII string
*/
_XF_API xfNChar*
xfConvertToASCII(xfChar *src,xfNChar *dst)
{
 xfInt i;

 for (i = 0; src[i]; i++)
  {
   if (!(src[i] & 0xFF80)) dst[i] = (xfNChar)src[i];
   else dst[i] = (xfNChar)'?';
  }

 dst[i] = '\x00';

 return dst;
}

/*
  Convert ASCII string to XFLib UTF-16 string
*/
_XF_API xfChar*
xfConvertFromASCII(xfNChar *src,xfChar *dst)
{
 xfInt i;

 for (i = 0; src[i]; i++)
   dst[i] = (xfChar)src[i];

 dst[i] = (xfChar)0x0000;

 return dst;
}

/*
== TAKEN FROM UNICODE STANDARD VERSION 4 ==

   Scalar Value                  1st Byte 2nd Byte 3rd Byte 4th Byte
   00000000 0xxxxxxx =>          0xxxxxxx
   00000yyy yyxxxxxx =>          110yyyyy 10xxxxxx
   zzzzyyyy yyxxxxxx =>          1110zzzz 10yyyyyy 10xxxxxx
   000uuuuu zzzzyyyy yyxxxxxx => 11110uuu 10uuzzzz 10yyyyyy 10xxxxxx
*/

/*
  Convert XFLib UTF-16 string to UTF-8 string
*/
_XF_API xfNChar*
xfConvertToUTF8(xfChar *src,xfNChar *dst)
{
 int i,c;

 for (c = i = 0; src[i]; i++)
  {
   /* 00000000 0xxxxxxxx (ASCII codes) */
   if (!(src[i] & 0xFF80))
    {
     dst[c] = (xfChar)src[i];
     c++;
     continue;
    }

   /* 00000yyy yyxxxxxx */
   if (!(src[i] & 0xF800))
    {
     dst[c]     = 0xC0 | (src[i] >> 6);
     dst[c + 1] = 0x80 | (src[i] & 0x003F);
     c += 2;
     continue;
    }

   /* zzzzyyyy yyxxxxxx */
   dst[c]     = 0xE0 | (src[i] >> 12);
   dst[c + 1] = 0x80 | ((src[i] >> 6) & 0x003F);
   dst[c + 1] = 0x80 | (src[i] & 0x003F);
   c += 3;
   continue;
  }

 dst[c] = '\x00';

 return dst;
}

/*
  Convert UTF-8 string to XFLib UTF-16 string
*/
_XF_API xfChar*
xfConvertFromUTF8(xfNChar *src,xfChar *dst)
{
 int i,c;

 for (c = i = 0; src[i] != '\x00'; c++)
  {
   /* 0xxxxxxx (ASCII codes) */
   if ((xfChar)(src[i] & (xfChar)0x80) == (xfChar)0x00)
    {
     dst[c] = (xfChar)src[i];
     i++;
     continue;
    }

   /* 110yyyyy 10xxxxxx */
   if ((xfChar)(src[i] & 0xE0) == (xfChar)(0xC0))
    {
     if ((xfChar)src[i + 1] == (xfChar)'\x00')
      { /* Incorrect UTF-8 string */ break; }
     if ((xfChar)(src[i + 1] & (xfChar)0xC0) != (xfChar)0x80)
      { /* Incorrect UTF-8 string */ continue; }
     dst[c] = ((src[i] & 0x1F) << 6) | (src[i + 1] & 0x3F);
     i += 2;
     continue;
    }

   /* 1110zzzz 10yyyyyy 10xxxxxx */
   if ((xfChar)(src[i] & 0xF0) == (xfChar)(0xE0))
    {
     if ((xfChar)src[i + 1] == (xfChar)'\x00')
      { /* Incorrect UTF-8 string */ break; }
     if ((xfChar)(src[i + 2]) == (xfChar)'\x00')
      { /* Incorrect UTF-8 string */ break; }
     if ((xfChar)(src[i + 1] & (xfChar)0xC0) != (xfChar)0x80)
      { /* Incorrect UTF-8 string */ continue; }
     if ((xfChar)(src[i + 2] & (xfChar)0xC0) != (xfChar)0x80)
      { /* Incorrect UTF-8 string */ continue; }

     dst[c] = ((src[i]     & 0x0F) << 12) | ((src[i + 1] & 0x3F) << 6)  |
               (src[i + 2] & 0x3F);
     i += 3;
     continue;
    }

   /* Incorrect UTF-8 string */
   i++;
   continue;
  }

 dst[c] = (xfChar)0x0000;

 return dst;
}

/*
  Swap bytes in wide char
*/
_XF_API xfChar
xfCharByteSwap(xfChar c)
{
 xfChar t;
 union
  {
   xfNChar c[2];
   xfChar wc;
  } s;

 s.wc = c;
 t = s.c[0];
 s.c[0] = s.c[1];
 s.c[1] = t;

 return s.wc;
}

/*
  Convert integer to decimal string
*/
_XF_API xfChar*
xfIntToDec(xfInt n,xfChar *str)
{
 int i,c = 0;
 xfChar dt[] = {L'0', L'1', L'2', L'3', L'4', L'5', L'6', L'7', L'8', L'9'};
 xfChar t;
 xfChar *rs;

 if (n < 0) { str[c] = L'-'; c++; n = -n; }
 if (!n) { str[c] = L'0'; c++; }

 while (n)
  {
   str[c] = dt[n % 10];
   c++;
   n /= 10;
  }
 str[c] = 0x0000;

 rs = str; if (str[0] == L'-') { rs++; c--; }
 c--;
 for (i = 0; i < (c >> 1); i++)
  {
   t = rs[i];
   rs[i] = rs[c - i];
   rs[c - i] = t;
  }

 return str;
}

/*
  Convert integer to hexadecimal string
*/
_XF_API xfChar*
xfIntToHex(xfInt n,xfChar *str)
{
 int i,c = 0;
 xfChar dt[] = {L'0', L'1', L'2', L'3', L'4', L'5', L'6', L'7',
                L'8', L'9', L'a', L'b', L'c', L'd', L'e', L'f'};
 xfChar t;
 xfChar *rs;

 str[c] = L'0'; c++;
 str[c] = L'x'; c++;
 if (!n) { str[c] = L'0'; c++; }

 while (n)
  {
   str[c] = dt[n & 0xF];
   c++;
   n = ((unsigned)n) >> 4;
  }
 str[c] = 0x0000;

 rs = str; rs += 2;
 c -= 3;
 for (i = 0; i < (c >> 1); i++)
  {
   t = rs[i];
   rs[i] = rs[c - i];
   rs[c - i] = t;
  }

 return str;
}

/*
  Convert decimal string to integer
*/
_XF_API xfInt
xfDecToInt(xfChar *str)
{
 xfInt n = 0,neg = 0;

 if (!str) return 0;
 if (!*str) return 0;

 if (*str == L'-') { neg = 1; str++; }
 while (*str)
  {
   if (*str < L'0' || *str > L'9') return 0;
   n *= 10;
   n += *str - L'0';
   str++;
  }

 if (neg) n = -n;

 return n;
}

/*
  Convert hexadecimal string to integer
*/
_XF_API xfInt
xfHexToInt(xfChar *str)
{
 xfInt n = 0;

 if (!str) return 0;
 if (*str != L'0') return 0;
 str++;
 if (*str != L'x' && *str != L'X') return 0;
 str++;

 while (*str)
  {
   if ((*str < L'0' || *str > L'9') &&
       (*str < L'A' || *str > L'F') &&
       (*str < L'a' || *str > L'f')) return 0;
   n <<= 4;
   if (*str >= L'0' && *str <= L'9') n += *str - L'0';
   if (*str >= L'A' && *str <= L'F') n += *str + 10 - L'A';
   if (*str >= L'a' && *str <= L'f') n += *str + 10 - L'a';
   str++;
  }

 return n;
}

