/*
  XFLib - XF management library

  (C) Max Volkov, Sevastopol 2006-2008
  Mailto: max at xfhome dot org
*/

/*
  XF internal system API header file
*/

#include "xf.h"

#include "hash.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct
 {
  FILE *file;
  xfChar buffer[3];
  xfInt bufSize;
  xfInt bufPos;

  xfInt pos;
  xfChar *string;

  xfInt type;
  xfInt fileType;
  xfInt byteSwap;
 } xfReader;

typedef struct
 {
  FILE *file;
  xfChar buffer[5];

  xfChar *string;
  xfNChar *wstring;
  xfInt size;
  xfInt maxSize;
  xfInt bufSize;

  xfInt type;
  xfInt shift;

  /* Output options */
  xfInt o_charCodes;
 } xfWriter;

/* User-defined memory manager */
extern xfUserAlloc_t xfUserAlloc;
extern xfUserFree_t  xfUserFree;

/* XF locale information */
extern xfMap *xfMsgList;                /* Message list map */
extern xfNode *xfMsgPtr;                /* Message list node */
extern xfChar xfMsgListString[];        /* String with default message list */

/* XF document reader source types */
#define XF_READER_MEMORY        0
#define XF_READER_FILE          1

/* XF doucment reader file types (encodings) */
#define XF_READER_FILE_ASCII    0
#define XF_READER_FILE_UTF8     1
#define XF_READER_FILE_UTF16BE  2
#define XF_READER_FILE_UTF16LE  3

/* XF document writer types */
#define XF_WRITER_MEMORY        1
#define XF_WRITER_FILE_ASCII    2
#define XF_WRITER_FILE_UTF8     3
#define XF_WRITER_FILE_UTF16    4

extern xfInt xfAllocCounter;
extern xfInt xfFreeCounter;
extern xfInt xfAllocSum;

_XF_SYS xfNode* xfAllocNode();
_XF_SYS xfChar* xfSetError(xfMap *map,xfChar *error);

_XF_SYS xfReader* xfCreateReader(xfInt type,xfVoid *source);
_XF_SYS xfVoid xfDestroyReader(xfReader *reader);
_XF_SYS xfInt xfReaderGetSize(xfReader *reader);
_XF_SYS xfChar xfReaderInputChar(xfReader *reader);

_XF_SYS xfWriter* xfCreateWriter(xfInt type,xfVoid *dest,xfInt maxSize,xfInt charCodes);
_XF_SYS xfVoid xfDestroyWriter(xfWriter *writer);
_XF_SYS xfVoid xfWriterOutputString(xfWriter *writer,xfChar *string);

_XF_SYS xfInt xfParsePath(xfChar *pathName,xfChar *nodeName);
_XF_SYS xfInt xfSysParse(xfMap *map,xfReader *reader);
_XF_SYS xfInt xfDigitHexToInt(xfInt hex);

_XF_SYS xfInt xfWriteNode(xfWriter *writer,xfNode *node);
_XF_SYS xfInt xfSysGenerate(xfMap *map,xfWriter *writer);

_XF_SYS xfInt xfNeedToPack(xfChar *s);
_XF_SYS xfChar* xfPackString(xfChar *s, xfChar *ns,int charCodes);
_XF_SYS xfChar* xfUnpackString(xfChar *s);

_XF_SYS xfInt xfParsePath(xfChar *pathName,xfChar *nodeName);

_XF_SYS xfHash* xfHashAddNode(xfHash *hash,xfNode *node);
_XF_SYS xfHash* xfHashRemoveNode(xfHash *hash,xfNode *node);
_XF_SYS xfNode* xfHashFindNode(xfHash *hash,xfChar *name);
_XF_SYS xfNode* xfCreateNodeHash(xfNode *node,xfInt hashSize);

_XF_SYS xfInt xfSysParseSomething(xfMap *map,xfVoid *something,xfInt type);

#include "xf.h"

#ifdef __cplusplus
}
#endif
