/*
  XFLib - XF management library

  (C) Max Volkov, Sevastopol 2006-2008
  Mailto: max at xfhome dot org
*/

/*
  XF document generator
*/

#include "xf.h"
#include "xf_api.h"
#include "xf_sys.h"

#include <stdio.h>
#include <string.h>

_XF_API xfInt
xfWriteFile(xfMap *map, xfNChar *filename)
{
 xfWriter *w;
 xfInt res,type;
 xfInt enc;

 type = XF_WRITER_FILE_ASCII;
 enc = xfGetDocEncoding(map);

 if (enc)
  {
   if (enc == XF_ENC_ASCII)   type = XF_WRITER_FILE_ASCII;
   if (enc == XF_ENC_UTF8)    type = XF_WRITER_FILE_UTF8;
   if (enc == XF_ENC_UTF16)   type = XF_WRITER_FILE_UTF16;
  }

 /* If encoding code is incorrect, we set it ti ASCII */
 if (type == XF_WRITER_FILE_ASCII) xfSetDocEncoding(map,XF_ENC_ASCII);

 w = xfCreateWriter(type,(xfVoid*)filename,0,map->o_charCodes);
 if (!w) return XFE_IO;

 res = xfSysGenerate(map,w);

 xfDestroyWriter(w);

 return res;
}

_XF_API xfInt
xfWriteString(xfMap *map, xfChar *string, xfInt maxSize)
{
 xfWriter *w;
 xfInt res;

 w = xfCreateWriter(XF_WRITER_MEMORY,(xfVoid*)string,maxSize,map->o_charCodes);
 if (!w) return XFE_IO;

 res = xfSysGenerate(map,w);

 xfDestroyWriter(w);

 return res;
}

_XF_SYS xfInt
xfWriteNode(xfWriter *writer,xfNode *node)
{
 xfNode *c;
 xfInt i;
 xfMap *m;
 xfChar *name;

 if (!node) return XF_NULL;
 if (node->name)
  {
   for (i = 0; i < writer->shift; i++) xfWriterOutputString(writer,XFS(" "));

   if (!xfNeedToPack(node->name))
    {
     xfWriterOutputString(writer,node->name);
    }
   else
    {
     /* Length(name) x 6 should be enough (for \xxxx; case) */
     name = (xfChar*)xfAlloc((xfStrlen(node->name) * 6 + 1) * sizeof(xfChar));
     name = xfPackString(node->name,name,writer->o_charCodes);
     xfWriterOutputString(writer,XFS("\""));
     xfWriterOutputString(writer,name);
     xfWriterOutputString(writer,XFS("\""));
     xfFree(name);
    }

   if (node->Class)
    {
     xfWriterOutputString(writer,XFS(": "));

     if (!xfNeedToPack(node->Class))
      {
       xfWriterOutputString(writer,node->Class);
      }
     else
      {
       /* Length(name) x 6 should be enough (for \xxxx; case) */
       name = (xfChar*)xfAlloc((xfStrlen(node->Class) * 6 + 1) * sizeof(xfChar));
       name = xfPackString(node->Class,name,writer->o_charCodes);
       xfWriterOutputString(writer,XFS("\""));
       xfWriterOutputString(writer,name);
       xfWriterOutputString(writer,XFS("\""));
       xfFree(name);
      }
    }

   if (node->value)
    {
     xfWriterOutputString(writer,XFS(" = "));

     if (!xfNeedToPack(node->value))
      {
       xfWriterOutputString(writer,node->value);
      }
     else
      {
       /* Length(name) x 6 should be enough (for \xxxx; case) */
       name = (xfChar*)xfAlloc((xfStrlen(node->value) * 6 + 1) * sizeof(xfChar));
       name = xfPackString(node->value,name,writer->o_charCodes);
       xfWriterOutputString(writer,XFS("\""));
       xfWriterOutputString(writer,name);
       xfWriterOutputString(writer,XFS("\""));
       xfFree(name);
      }
    }

   if (node->firstChild)
    {
     xfWriterOutputString(writer,XFS("\n"));
     for (i = 0; i < writer->shift + 1; i++) xfWriterOutputString(writer,XFS(" "));
     xfWriterOutputString(writer,XFS("{\n"));
     writer->shift += 2;
    }
   else
    {
     xfWriterOutputString(writer,XFS(";\n"));
     m = (xfMap*)node->map;
     if (node->parent == m->root) xfWriterOutputString(writer,XFS("\n"));
     return XFE_OK;
    }
  }

 c = node->firstChild;
 while (c)
  {
   xfWriteNode(writer,c);
   c = c->next;
  }

 if (node->name)
  {
   for (i = 0; i < writer->shift - 1; i++) xfWriterOutputString(writer,XFS(" "));
   xfWriterOutputString(writer,XFS("}\n"));
   writer->shift -= 2;
  }

 m = (xfMap*)node->map;
 if (node->parent == m->root) xfWriterOutputString(writer,XFS("\n"));

 return XFE_OK;
}

_XF_SYS xfInt
xfSysGenerate(xfMap *map,xfWriter *writer)
{
 xfInt i,ok;
 FILE *file;
 xfChar *enc, str[2] = {0x0000, 0x0000};

 if (map->o_addInfo)
  {
   xfWriterOutputString(writer,XFS("* Generated by "));
   xfWriterOutputString(writer,xfInfo());
   xfWriterOutputString(writer,XFS("\n\n"));
  }

 if (map->header)
   if (map->o_addComment)
    {
     xfWriterOutputString(writer,XFS("/*\n  "));
     for (i = 0; map->header[i]; i++)
      {
       ok = false;
       if (map->header[i + 1] == (xfChar)'/')
         if (map->header[i] == (xfChar)'*')
           { xfWriterOutputString(writer,XFS("/")); ok = true; }
       if (!ok) { str[0] = map->header[i]; xfWriterOutputString(writer,str); }
       if (map->header[i] == (xfChar)0x000A) xfWriterOutputString(writer,XFS("  "));
      }

     if (i)
       if (map->header[i] != 0x0A)
         xfWriterOutputString(writer,XFS("\n"));

     xfWriterOutputString(writer,XFS("*/\n\n"));
    }

 xfWriteNode(writer,map->root);

 return XFE_OK;
}


