/*
  XFLib sample
  Check XFLib security subsystem
*/

#include <stdio.h>

#include "./../../lib/xf.h"
#include "./../../lib/xf_api.h"

/* Max. number of legally possible nodes in the document */
#define SEC_LIMIT 100

xfSecurity *s;

void process(char *filename)
{
 xfMap *m;
 int r;

 m = xfCreate();
 xfSetDocSecurity(m,s);

 printf("reading file %s... ",filename);
 r = xfReadFile(m,filename);

 if (r) {
   if (r == XFE_IO) printf("cannot open file %s\n",filename);
   if (r == XFE_FORMAT) printf("parse error (file %s, line %i, col %i): %S\n",
                               filename,xfGetErrorLine(m),xfGetErrorCol(m),
                               xfGetError(m));
   return;
  }
 printf("Okay!!!\n");

 xfDestroy(m);
}

int main(int argc,char *argv[])
{
 int i;

 if (argc < 2) {
   printf("check XFLib security\n");
   printf("usage: security file1 file2 ...\n");
   printf("maximal number of possible nodes: %i\n",SEC_LIMIT);
  }

 xfStart();

 s = xfCreateSecurity();
 xfSecuritySetNodeLimit(s,SEC_LIMIT);

 for (i = 1; i < argc; i++)
    process(argv[i]);

 xfDestroySecurity(s);

 xfStop();

 return 0;
}


